#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ptrace.h>
#include <machine/reg.h>

#include "ltrace.h"

#if (!defined(PTRACE_PEEKUSER) && defined(PTRACE_PEEKUSR))
# define PTRACE_PEEKUSER PTRACE_PEEKUSR
#endif

#if (!defined(PTRACE_POKEUSER) && defined(PTRACE_POKEUSR))
# define PTRACE_POKEUSER PTRACE_POKEUSR
#endif

void get_arch_dep(struct process *proc)
{
}

/* Returns 1 if syscall, 2 if sysret, 0 otherwise.
 */
int syscall_p(struct process *proc, int status, int *sysnum)
{
	/*
	hard to know what to do here since PT_SYSCALL is undocumented
	*/
	if (WIFSTOPPED(status)
	    && WSTOPSIG(status) == (SIGTRAP | proc->tracesysgood)) {

		if (proc->callstack_depth > 0 &&
		    proc->callstack[proc->callstack_depth - 1].is_syscall) {
			/* 
			 * We have to assume that the sysret is for the
			 * most recent syscall, since apparently FreeBSD
			 * doesn't save the syscall number. But that's
			 * a reasonable assumption.
			 */
			*sysnum = proc->callstack[proc->callstack_depth - 1].c_un.syscall;
			return 2;
		}

		/*
		 * SIGTRAP can be raised by:
		 * (1) a breakpoint (used to catch library call entries and
		 * exit).
		 * (2) a single step (used to restore a breakpoint after
		 * temporarily disabling it)
		 * (3) a system call entry
		 * (4) a system call exit
		 * Case (4) is handled above -- we assume that after a
		 * system call entry it's the only possibility.
		 * Case (3) is checked by eliminating cases (1) and (2)
		 *
		 * So we'll check if the PC is on a breakpoint and
		 * that there's no breakpoint currently being enabled.
		 * (FIXME: this doesn't work when the first instruction
		 * in the library function issues a system call).
		 */		
		if (!proc->breakpoints || 
				(!proc->breakpoint_being_enabled &&
				 !address2bpstruct(proc, md_fetch_eip(proc->pid) - DECR_PC_AFTER_BREAK))) {
		*sysnum = md_fetch_eax(proc->pid);
			return 1;
		}
	}
	return 0;
}

long gimme_arg(enum tof type, struct process *proc, int arg_num)
{
	/* FIXME: doesn't work with SYS_syscall and SYS__syscall
	 * based syscalls since their arguments are in a different
	 * location (see i386/i386/trap.c:syscall() */
	if (arg_num == -1) {	/* return value */
		return md_fetch_eax(proc->pid);
	}

	if (type == LT_TOF_FUNCTION || type == LT_TOF_FUNCTIONR) {
		return ptrace(PTRACE_PEEKTEXT, proc->pid,
			      proc->stack_pointer + 4 * (arg_num + 1), 0);
	} else if (type == LT_TOF_SYSCALL || type == LT_TOF_SYSCALLR) {
		/* proc->stack_pointer isn't set here so don't use it */
		return ptrace(PT_READ_I, proc->pid, 
				md_fetch_esp(proc->pid) + 4 * (arg_num + 1), 0);
	} else {
		fprintf(stderr, "gimme_arg called with wrong arguments\n");
		exit(1);
	}

	return 0;
}

void save_register_args(enum tof type, struct process *proc)
{
}
